//-------------------------------------------------------------------------
// MainWindow
//-------------------------------------------------------------------------

#include "MainWindow.h"

BEGIN_EVENT_TABLE(MainWindow, wxFrame)
    EVT_MENU(SPLIT_VERTICAL, MainWindow::SplitVertical)
    EVT_MENU(SPLIT_HORIZONTAL, MainWindow::SplitHorizontal)
    EVT_MENU(SPLIT_UNSPLIT, MainWindow::Unsplit)
    EVT_MENU(SPLIT_LIVE, MainWindow::ToggleLive)
    EVT_MENU(SPLIT_SETPOSITION, MainWindow::SetPosition)
    EVT_MENU(SPLIT_SETMINSIZE, MainWindow::SetMinSize)

    EVT_MENU(SPLIT_QUIT, MainWindow::Quit)

    EVT_UPDATE_UI(SPLIT_VERTICAL, MainWindow::UpdateUIVertical)
    EVT_UPDATE_UI(SPLIT_HORIZONTAL, MainWindow::UpdateUIHorizontal)
    EVT_UPDATE_UI(SPLIT_UNSPLIT, MainWindow::UpdateUIUnsplit)
END_EVENT_TABLE()

// My frame constructor
MainWindow::MainWindow()
       : wxFrame(NULL, -1, _T("wxSplitterWindow sample"),
                 wxDefaultPosition, wxSize(420, 300),
                 wxDEFAULT_FRAME_STYLE | wxNO_FULL_REPAINT_ON_RESIZE)
{
    CreateStatusBar(2);

    // Make a menubar
    wxMenu *splitMenu = new wxMenu;
    splitMenu->Append(SPLIT_VERTICAL,
                      _T("Split &Vertically\tCtrl-V"),
                      _T("Split vertically"));
    splitMenu->Append(SPLIT_HORIZONTAL,
                      _T("Split &Horizontally\tCtrl-H"),
                      _T("Split horizontally"));
    splitMenu->Append(SPLIT_UNSPLIT,
                      _T("&Unsplit\tCtrl-U"),
                      _T("Unsplit"));
    splitMenu->AppendSeparator();

    splitMenu->AppendCheckItem(SPLIT_LIVE,
                               _T("&Live update\tCtrl-L"),
                               _T("Toggle live update mode"));
    splitMenu->Append(SPLIT_SETPOSITION,
                      _T("Set splitter &position\tCtrl-P"),
                      _T("Set the splitter position"));
    splitMenu->Append(SPLIT_SETMINSIZE,
                      _T("Set &min size\tCtrl-M"),
                      _T("Set minimum pane size"));
    splitMenu->AppendSeparator();

    splitMenu->Append(SPLIT_QUIT, _T("E&xit\tAlt-X"), _T("Exit"));

    wxMenuBar *menuBar = new wxMenuBar;
    menuBar->Append(splitMenu, _T("&Splitter"));

    SetMenuBar(menuBar);

    menuBar->Check(SPLIT_LIVE, TRUE);
    m_splitter = new MainWindowSplitter(this);

#if 1
    m_left = new FileSelector(m_splitter, true);
    m_left->SetBackgroundColour(*wxRED);
    m_left->SetScrollbars(20, 20, 5, 5);
    m_left->SetCursor(wxCursor(wxCURSOR_MAGNIFIER));

    m_right = new ImageMatrix(m_splitter, false);
    m_right->SetBackgroundColour(*wxCYAN);
    m_right->SetScrollbars(20, 20, 5, 5);
#else // for testing kbd navigation inside the splitter
    m_left = new wxTextCtrl(m_splitter, -1, _T("first text"));
    m_right = new wxTextCtrl(m_splitter, -1, _T("second text"));
#endif

    // you can also do this to start with a single window
#if 0
    m_right->Show(FALSE);
    m_splitter->Initialize(m_left);
#else
    // you can also try -100
    m_splitter->SplitVertically(m_left, m_right, 100);
#endif

    SetStatusText(_T("Min pane size = 0"), 1);
}

MainWindow::~MainWindow()
{
}

// menu command handlers

void MainWindow::Quit(wxCommandEvent& WXUNUSED(event) )
{
    Close(TRUE);
}

void MainWindow::SplitHorizontal(wxCommandEvent& WXUNUSED(event) )
{
    if ( m_splitter->IsSplit() )
        m_splitter->Unsplit();
    m_left->Show(TRUE);
    m_right->Show(TRUE);
    m_splitter->SplitHorizontally( m_left, m_right );

    SetStatusText(_T("Splitter split horizontally"), 1);
}

void MainWindow::SplitVertical(wxCommandEvent& WXUNUSED(event) )
{
    if ( m_splitter->IsSplit() )
        m_splitter->Unsplit();
    m_left->Show(TRUE);
    m_right->Show(TRUE);
    m_splitter->SplitVertically( m_left, m_right );

    SetStatusText(_T("Splitter split vertically"), 1);
}

void MainWindow::Unsplit(wxCommandEvent& WXUNUSED(event) )
{
    if ( m_splitter->IsSplit() )
        m_splitter->Unsplit();
    SetStatusText(_T("No splitter"));
}

void MainWindow::ToggleLive(wxCommandEvent& event )
{
    long style = m_splitter->GetWindowStyleFlag();
    if ( event.IsChecked() )
        style |= wxSP_LIVE_UPDATE;
    else
        style &= ~wxSP_LIVE_UPDATE;

    m_splitter->SetWindowStyleFlag(style);
}

void MainWindow::SetPosition(wxCommandEvent& WXUNUSED(event) )
{
    wxString str;
    str.Printf( wxT("%d"), m_splitter->GetSashPosition());
    str = wxGetTextFromUser(_T("Enter splitter position:"), _T(""), str, this);
    if ( str.empty() )
        return;

    long pos;
    if ( !str.ToLong(&pos) )
    {
        wxLogError(_T("The splitter position should be an integer."));
        return;
    }

    m_splitter->SetSashPosition(pos);

    wxLogStatus(this, _T("Splitter position set to %ld"), pos);
}

void MainWindow::SetMinSize(wxCommandEvent& WXUNUSED(event) )
{
    wxString str;
    str.Printf( wxT("%d"), m_splitter->GetMinimumPaneSize());
    str = wxGetTextFromUser(_T("Enter minimal size for panes:"), _T(""), str, this);
    if ( str.empty() )
        return;

    int minsize = wxStrtol( str, (wxChar**)NULL, 10 );
    m_splitter->SetMinimumPaneSize(minsize);
    str.Printf( wxT("Min pane size = %d"), minsize);
    SetStatusText(str, 1);
}

// Update UI handlers

void MainWindow::UpdateUIHorizontal(wxUpdateUIEvent& event)
{
    event.Enable( (!m_splitter->IsSplit()) || (m_splitter->GetSplitMode() != wxSPLIT_HORIZONTAL) );
}

void MainWindow::UpdateUIVertical(wxUpdateUIEvent& event)
{
    event.Enable( ( (!m_splitter->IsSplit()) || (m_splitter->GetSplitMode() != wxSPLIT_VERTICAL) ) );
}

void MainWindow::UpdateUIUnsplit(wxUpdateUIEvent& event)
{
    event.Enable( m_splitter->IsSplit() );
}
